# -*- coding: utf-8 -*-
"""Module with resources to model a bibliography."""
from .bibliography import Bibliography
from .entry import BibliographyEntry

__all__ = ('Bibliography', 'BibliographyEntry')
