{% block content %}
{% if form %}
Upload a new bibliographic entry to the bibliography:
<form action="{% url 'upload-entry' %}" method="post" enctype="multipart/form-data">
    {% csrf_token %}
    {{ form.as_p }}
    <input type="submit" value="Submit">
</form>
{% else %}
<p>Uploading of entries is disabled because no file storage has been configured for the bibliography.</p>
{% endif %}
{% endblock %}
